#ifndef OP5_LICENSE_ENV_H_
#define OP5_LICENSE_ENV_H_
enum OP5LIC_ENV_TYPE {
	ENV_TYPE_INTEGER = 0,
	ENV_TYPE_ENV_LIST,
	ENV_TYPE_STRING
};

struct op5lic_env_list_t;

struct op5lic_env_node_t {
	char *key;
	enum OP5LIC_ENV_TYPE type;
	union {
		int val_int;
		struct op5lic_env_list_t *val_env_list;
		char * val_string;
	} value;
};

/*
 * frees a tokenized path, as returned by op5lic_env_tokenize_path()
 */
void op5lic_env_free_tokenized_path(char **path, int depth);

/*
 * tokenizes a environment tree path of the form "foo.bar.baz" into its
 * constituent elements. The `depth' parameter is updated by the function
 * to contain the depth of the path, which is also the number of string
 * elements in the returned array.
 *
 * NOTE: Caller is responsible for calling op5lic_env_free_tokenized_path() on
 * the resulting array.
 */
char ** op5lic_env_tokenize_path(const char *nodepath, int *depth);


/*
 * constructor/destructor pair
 */
struct op5lic_env_list_t *op5lic_env_list_create(void);
void op5lic_env_list_destroy(struct op5lic_env_list_t *);

/*
 * Adds a new node to an environment list.
 *
 * NOTE: This function transfers ownership of `new_node', it is an error to free new_node
 * after calling this function.
 */
void op5lic_env_list_add(struct op5lic_env_list_t *, struct op5lic_env_node_t *new_node);

/*
 * Extracts the environment list from `node', if the type of the node is
 * ENV_TYPE_ENV_LIST and returns the list. Calling this function destroys the
 * original node. If node isn't of type ENV_TYPE_ENV_LIST, NULL is returned
 * (but the node is still destroyed).
 */
struct op5lic_env_list_t * op5lic_env_node_to_list(struct op5lic_env_node_t *node);

/*
 * Returns a (heap-allocated) string representation of the supplied `list'. The
 * caller is responsible for free()ing the returned string.
 */
char * op5lic_env_list_to_string(const struct op5lic_env_list_t *list);
/*
 * Finds the node with path `path' in an environment list, if it exists, or
 * returns NULL if such a node does not exist.
 */
struct op5lic_env_node_t * op5lic_env_list_find(const struct op5lic_env_list_t *, const char *path);

/*
 * Creates a new `integer' environment node
 */
struct op5lic_env_node_t * op5lic_env_create_int(const char *key, int value);
/*
 * Creates a new `environment list' environment node
 */
struct op5lic_env_node_t * op5lic_env_create_list(const char *key);

/*
 * Creates a new `string' environment node
 */
struct op5lic_env_node_t * op5lic_env_create_string(const char *key, const char *value);

/*
 * Destroys an environment node.
 *
 * NOTE: In most cases, op5lic_env_list_destroy() is probably what you want.
 */
void op5lic_env_node_destroy(struct op5lic_env_node_t *);

#endif
